/*
	WOAccessConfig.h
    WOAccess Framework Release 1.0 beta
    Copyright (c) 1997, Starr.Com.  All rights reserved.
*/

#import <Foundation/NSObject.h>
#import "WOAccessProtocols.h"
#import "WOAccessTypes.h"

@class NSString;
@class NSArray;
@class WOAccessMode;
@class NSDictionary;

@interface WOAccessConfig:NSObject
{
	NSString		*pageName;
	NSDictionary	*configuration;
	@private
	NSArray			*accessOrder;
	WOAccessMode	*defaultAccessMode;
	id <NSObject,WOAccessAuthenticator> authenticator;
	id <NSObject, WOAccessCertifier> certifier;
	NSArray			*userAuthOrder;
	NSArray			*groupAuthOrder;
	NSArray			*realmMembers;
	NSString		*realmName;
	unsigned int	_override;
	BOOL			_authUsersFirst;
}

+ (WOAccessConfig*) accessWithPath:(NSString*) aPath
	pageName:(NSString*) aName;
+ (WOAccessConfig*) accessWithConfig:(NSDictionary*) aConfig
	pageName:(NSString*) aName;

- initWithConfig:(NSDictionary*) aConfig
	pageName:(NSString*) aName;

- (NSString*) pageName;
- (NSDictionary*) configuration;

- (WOAccessMode*) defaultAccessMode;
- (void) setDefaultAccessMode:(WOAccessMode*) aMode;

- (NSArray*) accessOrder;

- (id <WOAccessAuthenticator>) authenticator;
- (id <WOAccessCertifier>) certifier;

- (NSArray*) realmMembers;
- (NSString*) realmName;

- (NSArray*) userAuthOrder;
- (NSArray*) groupAuthOrder;

- (BOOL) allowsOverride;
- (BOOL) allowsAccessOverride;
- (BOOL) allowsAuthenticationOverride;
- (BOOL) allowsCertificationOverride;
- (BOOL) allowsUsersOverride;
- (BOOL) allowsGroupsOverride;
- (void) setAllowsOverride:(BOOL) state;
- (void) setAllowsAccessOverride:(BOOL) state;
- (void) setAllowsAuthenticationOverride:(BOOL) state;
- (void) setAllowsCertificationOverride:(BOOL) state;
- (void) setAllowsUsersOverride:(BOOL) state;
- (void) setAllowsGroupsOverride:(BOOL) state;

@end

@interface WOAccessConfig(AccessControl)
- (WOAccessStatusCode) checkAccessForUserNamed:(NSString*) userName
	inContext:(WOAccessContext*) aContext;
- (WOAccessStatusCode) checkAccessForGroupNamed:(NSString*) userName
	inContext:(WOAccessContext*) aContext;
@end


@interface WOAccessPageConfig:WOAccessConfig
{
	WOAccessConfig *defaultAccess;
}

+ (WOAccessConfig*) accessWithPath:(NSString*) aPath
	pageName:(NSString*) aName
	default:(WOAccessConfig*) aDefault;

+ (WOAccessConfig*) accessWithConfig:(NSDictionary*) aConfig
	pageName:(NSString*) aName
	default:(WOAccessConfig*) aDefault;

- (void) setDefaultAccess:(WOAccessConfig*) aDefault;
- (WOAccessConfig*) defaultAccess;
- initWithConfig:(NSDictionary*) aConfig
	pageName:(NSString*) aName
	default:(WOAccessConfig*) aDefault;

@end