/*
    WOAccessProtcols.h
    WOAccess Framework Release 1.0 beta
    Copyright (c) 1997, Starr.Com.  All rights reserved.
*/

#import <Foundation/NSObject.h>

@class NSDictionary;
@class NSMutableDictionary;
@class NSString;
@class NSArray;
@class WOAccessError;
@class WOAccessContext;
@class WOComponent;

@protocol WOAccessAuthenticator

- initWithConfiguration:(NSDictionary*) config;
- (WOAccessError*) authenticateWithContext:(WOAccessContext*) aContext;
- (NSDictionary*) configuration;

@end

@protocol WOAccessCertifier

- initWithConfiguration:(NSDictionary*) config;
- (WOAccessError*) certifyAccessForCredentials:(NSDictionary*) someCredentials
    inContext:(WOAccessContext*) aContext;
- (BOOL) isUser:(NSString*) userName memberOf:(NSString*) groupName;
- (NSDictionary*) configuration;

@end

@protocol WOAccessLoginComponent

// The page the user was denied access to
- (void) setNextPage:(WOComponent*) component;
- (WOComponent*) nextPage;
- (void) setNextPageName:(NSString*) component;
- (NSString*) nextPageName;

// Action to submit credentials
- (WOComponent*) login;

// The credentials, depends on certifier
// Usually UserName & Password
// Is stored as 'WOAccessCredentials' in the session
- (NSMutableDictionary*) credentials;
- (void) setCredentials:(NSMutableDictionary*) someCredentials;

// Optional realm, is used instead of page name in login text
- (void) setRealm:(NSString*) realm;
- (NSString*) realm;

// An error message to display
- (void) setErrorMessage:(NSString*) msg;
- (NSString*) errorMessage;

@end

