#import <WebObjects/WebObjects.h>
#import <WOAccess/WOAccess.h>
#import "ApacheCertifier.h"
#import "mod_auth.h"

/*
	Really inefficient certfier.
	Reads in user file once per certification
	And group file for each group allowed access.

	But it is compatible to the mod_auth.c from the apache server.

	The user file is in the format:

	user_name:crypted_pw
	
	The group file in the format:

	group_name: user_a user_b ...

*/

@implementation ApacheCertifier

- initWithConfiguration:(NSDictionary*) aConfig
{
	[super init];
	if(aConfig && [aConfig isKindOfClass:[NSDictionary class]])
	{
		configuration = [aConfig retain];
		return self;
	}
	NSLog(@"%@: Invalid configuration", [self class] );
	[self release];
	return nil;
}

- (WOAccessError*) certifyAccessForCredentials:(NSDictionary*) someCredentials
    inContext:(WOAccessContext*) aContext
{
    NSString *sent_pw, *real_pw;
	NSString *userName;
    NSString *auth_pwfile;
	WOAccessConfig	*access;

    if(!(auth_pwfile = [configuration objectForKey:@"AuthUserFile"]) )
    {
        return [WOAccessError accessError:@"Not Authorized"
                                    reason:@"Can't locate user file"
                                     info:configuration];
    }

    userName = [someCredentials objectForKey:@"User"];
    if (!(real_pw = get_pw(userName, auth_pwfile)))
    {
        return [WOAccessError accessError:@"Not Authorized"
                                   reason:@"Unknown user"
                                    info:someCredentials];
    }
    sent_pw = [someCredentials objectForKey:@"Password"];
	//
    sent_pw = WOAccessCryptPassword(sent_pw, real_pw);	
    /* anyone know where the prototype for crypt is? */
    if([sent_pw isEqualToString:real_pw])
	{
        // Get current config
        access = [aContext accessConfig];
        // Check User
        // Checks if user is allowed access, or any of the groups the user belongs to
        switch([access checkAccessForUserNamed:userName inContext:aContext])
        {
            case WOAccessAllowed:
                #ifdef DEBUG
                NSLog(@"%@: User Access allowed", [self class]);
                #endif
                return nil;
            case WOAccessDenied:
                #ifdef DEBUG
                NSLog(@"%@: User Access denied", [self class]);
                #endif
                [[(WOAccessApplication*)[WOApplication application] accessManager] deny];
                return nil;
            default: // Not in access list
                break;
        }
        return [WOAccessError accessError:@"Not Authorized"
                                    reason:@"User not in access list"
                                    info:someCredentials];
    }
    else
    {
        return [WOAccessError accessError:@"Not Authorized"
                                reason:@"Wrong password"
                                info:someCredentials];
    }

    return nil;
}

- (BOOL) isUser:(NSString*) userName
    memberOf:(NSString*) groupName
{
    NSSet	*groups;
	NSString	*auth_grpfile;

    if(!(auth_grpfile = [configuration objectForKey:@"AuthGroupFile"]) )
    {
        NSLog(@"%@: Can't find group file", [self class]);
		return NO;
    }
    groups = groups_for_user(userName,auth_grpfile);

    if(groups && [groups member:groupName])
    {
		return YES;
    }
    return NO;
}

- (NSDictionary*) configuration
{
    return configuration;
}

- (void) dealloc
{
    [configuration release];
    [super dealloc];
}

@end
