/* ====================================================================
 * Copyright (c) 1995 The Apache Group.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission.
 *
 * 5. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Group and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Group and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

#import <stdio.h>
#import <stdlib.h>
#import <string.h>
#import <Foundation/Foundation.h>
#import "mod_auth.h"

/*
 * http_auth: authentication
 * 
 * Rob McCool
 * 
 * Adapted to Shambhala by rst.
 * 
 * Hacked to work with Foundation Framework by MMF
 */

int cfg_getline(char *s, int n, FILE *f);
NSString *getword(char **line, char stop);
NSString *getword_conf(char **line);

#define MAX_STRING_LEN 2048

NSString *get_pw(NSString *user, NSString *auth_pwfile)
{
    FILE *f;
    char l[MAX_STRING_LEN];
    char *rpw;
	NSString *w;

	// There was something about fopen not working in DLL's 
    if(!(f=fopen( [auth_pwfile cString], "r")))
	{
        NSLog(@"Could not open password file: %@", auth_pwfile);
		return nil;
    }

    while(!(cfg_getline(l,MAX_STRING_LEN,f)))
	{
        if((l[0] == '#') || (!l[0]))
		{
			continue;
		}
		rpw = l;
        w = getword(&rpw, ':');
        if([user isEqual:w])
		{
	    	fclose(f);
            return [NSString stringWithCString:rpw];
		}
    }
    fclose(f);
    return nil;
}

NSSet *groups_for_user (NSString *user, NSString *grpfile)
{
    FILE *f;
    NSMutableSet *grps;
    NSAutoreleasePool *sp;
    char l[MAX_STRING_LEN];
    char *ll;
    NSString *w;
    NSString *group_name;

    if(!(f=fopen( [grpfile cString], "r")))
	{
        return nil;
	}

	grps = [NSMutableSet setWithCapacity:5];
    sp = [[NSAutoreleasePool alloc] init];
    
    while(!(cfg_getline(l,MAX_STRING_LEN,f)))
	{
		if((l[0] == '#') || (!l[0])) continue;
		ll = l;
	
        group_name = getword(&ll, ':');

		while(ll[0])
		{
            w = getword_conf (&ll);
            if([w isEqualToString:user])
			{
            	//table_set (grps, group_name, "in");
                [grps addObject:group_name];
           		break;
	    	}
		}
    }
    fclose( f);
    [sp release];
    return (NSSet*) grps;
}

#define CR '\r'
#define LF '\n'
int
cfg_getline(char *s, int n, FILE *f)
 {
    register int i=0, c;

    s[0] = '\0';
    /* skip leading whitespace */
    do {
        c = getc(f);
    } while (c == '\t' || c == ' ');

    while(1) {
        if((c == '\t') || (c == ' ')) {
            s[i++] = ' ';
            while((c == '\t') || (c == ' '))
                c = getc(f);
        }
        if(c == CR) {
            c = getc(f);
        }
        if(c == EOF || c == 0x4 || c == LF || i == (n-1)) {
            /* blast trailing whitespace */
            while(i && (s[i-1] == ' ')) --i;
            s[i] = '\0';
            return (feof(f) ? 1 : 0);
        }
        s[i] = c;
        ++i;
        c = getc(f);
    }
}
   

/* Get a word, (new) config-file style --- quoted strings and backslashes
 * all honored
 */

NSString *substring_conf (char *start, int len);
NSString *substring_conf (char *start, int len)
{
    char *result;
    char *resp;
    int i;

	resp = result = malloc(len + 2);

    for (i = 0; i < len; ++i)
	{
        if (start[i] == '\\') 
		{
			*resp++ = start[++i];
		}
        else
        {
            *resp++ = start[i];
        }
	}

    *resp = '\0';
	
    return [[[NSString alloc] initWithCStringNoCopy:result
				length: resp-result
				freeWhenDone:YES] autorelease];
}

NSString *getword_conf(char **line)
{
    char *str = *line, *strend;
    char quote;
	NSString *res;

    while (*str && isspace (*str))
        ++str;

    if (!*str)
	{
        *line = str;
        return @"";
    }

    if ((quote = *str) == '"' || quote == '\'')
	{
		strend = str + 1;
		while (*strend && *strend != quote) {
			if (*strend == '\\' && strend[1]) strend += 2;
			else ++strend;
		}
	
		res = substring_conf (str + 1, strend - str - 1);
	
		if (*strend == quote)
		{
			++strend;
		}
    } 
	else {
        strend = str;
	while (*strend && !isspace (*strend))
	    if (*strend == '\\' && strend[1]) strend += 2;
	    else ++strend;

		res = substring_conf (str, strend - str);
    }

    while (*strend && isspace(*strend)) ++ strend;
    *line = strend;
    return res;
}

int ind(const char *s, char c);
int ind(const char *s, char c)
{
    register int x;

    for(x=0;s[x];x++)
        if(s[x] == c) return x;

    return -1;
}

NSString *getword(char **line, char stop)
{
    int pos = ind(*line, stop);
    NSString *res;

    if (pos == -1)
	{
        res = [NSString stringWithCString:*line];
		*line += strlen (*line);
		return res;
    }
  
	res = [NSString stringWithCString:*line length:pos];
    
    while ((*line)[pos] == stop) ++pos;
    
    *line += pos;
    
    return res;
}

